import QueryString from 'querystring';
import PTUtil from '../utils/pt-util';

const {
  openNotification, getUrlPrefix, buildRequest, HttpRequest, isFetchCodeNice,
} = PTUtil;
const { GET } = HttpRequest;
const URL_PREFIX = getUrlPrefix();

const GET_TECHNOLOGIES = buildRequest('GET_TECHNOLOGIES');
const getTechnologies = type => (dispatch) => {
  dispatch({
    type: GET_TECHNOLOGIES.REQUEST,
  });
  const params = QueryString.stringify({ type });
  return GET()
    .setUrl(`${URL_PREFIX}technology/list?${params}`)
    .dofetch()
    .then((result) => {
      const { meta, data } = result;
      if (isFetchCodeNice(meta.code)) {
        dispatch({
          type: GET_TECHNOLOGIES.SUCCESS,
          data,
        });
        return data;
      }
      dispatch({
        type: GET_TECHNOLOGIES.FAIL,
      });
      openNotification('error', meta.code, 0, meta.message);
      return false;
    })
    .catch((e) => {
      dispatch({
        type: GET_TECHNOLOGIES.FAIL,
      });
      openNotification('error', '', 0, e.message);
    });
};

const GET_TECHNOLOGY = buildRequest('GET_TECHNOLOGY');
const getTechnology = name => (dispatch) => {
  dispatch({
    type: GET_TECHNOLOGY.REQUEST,
  });
  return GET()
    .setUrl(`${URL_PREFIX}technology/${name}`)
    .dofetch()
    .then((result) => {
      const { meta, data } = result;
      if (isFetchCodeNice(meta.code)) {
        dispatch({
          type: GET_TECHNOLOGY.SUCCESS,
          data,
          name,
        });
        return data;
      }
      dispatch({
        type: GET_TECHNOLOGY.FAIL,
      });
      openNotification('error', meta.code, 0, meta.message);
      return false;
    })
    .catch((e) => {
      dispatch({
        type: GET_TECHNOLOGY.FAIL,
      });
      openNotification('error', '', 0, e.message);
    });
};

export {
  GET_TECHNOLOGIES,
  GET_TECHNOLOGY,
  getTechnologies,
  getTechnology,
};
