import { notification } from 'antd';
import { fromJS, is } from 'immutable';
import * as tool from 'lodash';
import { HTTP_METHOD, TOKEN } from '../constants/const';
import FetchUtil from './fetch-util';

const SESSION = {
  set: (key, value) => {
    sessionStorage.setItem(key, value);
  },
  get: key => sessionStorage.getItem(key),
  delete: (key) => {
    sessionStorage.removeItem(key);
  },
};

const LOCAL = {
  set: (key, value) => {
    localStorage.setItem(key, value);
  },
  get: key => localStorage.getItem(key),
  delete: (key) => {
    localStorage.removeItem(key);
  },
};

const getHeaders = () => ({
  'Content-Type': 'application/json;charset=utf-8',
  Authorization: SESSION.get(TOKEN),
});

const HttpRequest = {
  GET: () => {
    const fetchUtil = new FetchUtil();
    return fetchUtil
      .init()
      .setMethod(HTTP_METHOD.GET)
      .setHeader(getHeaders());
  },
  POST: () => {
    const fetchUtil = new FetchUtil();
    return fetchUtil
      .init()
      .setMethod(HTTP_METHOD.POST)
      .setBodyType('json')
      .setHeader(getHeaders());
  },
  PUT: () => {
    const fetchUtil = new FetchUtil();
    return fetchUtil
      .init()
      .setMethod(HTTP_METHOD.PUT)
      .setBodyType('json')
      .setHeader(getHeaders());
  },
  DELETE: () => {
    const fetchUtil = new FetchUtil();
    return fetchUtil
      .init()
      .setMethod(HTTP_METHOD.DELETE)
      .setHeader(getHeaders());
  },
};

const PTUtil = {
  each: tool.each,
  map: tool.map,
  isObject: tool.isObject,
  isNumber: tool.isNumber,
  isString: tool.isString,
  isFunction: tool.isFunction,
  isFinite: tool.isFinite,
  isBoolean: tool.isBoolean,
  isEmpty: tool.isEmpty,
  lowerFirst: tool.lowerFirst,
  upperFirst: tool.upperFirst,
  upperCase: tool.upperCase,
  isNil: tool.isNil,
  isNull: tool.isNull,
  isArray: tool.isArray,
  isDate: tool.isDate,
  isPlainObject: tool.isPlainObject,
  toArray: tool.toArray,
  indexOf: tool.indexOf,
  assign: tool.assign,
  groupBy: tool.groupBy,
  cloneDeep: tool.cloneDeep,
  maxBy: tool.maxBy,
  minBy: tool.minBy,
  round: tool.round,
  filter: tool.filter,
  isEqualWith: tool.isEqualWith,
  isEqual: tool.isEqual,
  replace: tool.replace,
  union: tool.union,
  pick: tool.pick,
  uniq: tool.uniq,
  uniqBy: tool.uniqBy,
  trim: tool.trim,
  isUndefined: tool.isUndefined,
  sortBy: tool.sortBy,
  is: (first, second) => (is(fromJS(first), fromJS(second))),
  isFetchCodeNice: code => /^SC_2[0-6]{2}$/.test(code),
  SESSION,
  LOCAL,
  HttpRequest,
  getUrlPrefix: () => {
    const env = process.env.NODE_ENV;
    if (env.indexOf('server') !== -1) {
      return 'http://localhost:7070/';
    }
    return '/';
  },
  openNotification(type, title, duration, description, key) {
    notification[type]({
      message: PTUtil.isEmpty(title) ? '提示信息' : title,
      description,
      duration,
      key,
      style: {
        marginTop: '50px',
      },
    });
  },
  redirect: (meta) => {
    if (
      meta.code === 'SC_405'
      && (meta.message === 'token expired.' || meta.message === 'invalid token.')
    ) {
      window.location.href = '/#/login';
      return true;
    }
    return false;
  },
  buildRequest: prefix => ({
    REQUEST: `${prefix}_REQUEST`,
    SUCCESS: `${prefix}_SUCCESS`,
    FAIL: `${prefix}_FAIL`,
  }),
};

export default PTUtil;
