const fs = require('fs');
const path = require('path');
const { prompt } = require('inquirer');
const questions = [
  {
    type: 'input',
    name: 'name',
    message: 'name:',
  },
  {
    type: 'input',
    name: 'version',
    message: 'version:',
  },
  {
    type: 'input',
    name: 'description',
    message: 'description:',
  },
  {
    type: 'input',
    name: 'keywords',
    message: 'keywords:',
  },
  {
    type: 'input',
    name: 'author',
    message: 'author:',
  }
];

module.exports = function(appPath, projectName) {
  const tplPath = path.resolve(appPath, projectName, './package.json');
  const tplJson = require(tplPath);

  return new Promise(function (resolve) {
    console.log('\nConfiguring the package.json file, input and enter:\n');
    prompt(questions).then(function(data) {
      const { name, version, description, keywords, author } = data;
      tplJson.name = name ||  tplJson.name;
      tplJson.version = version || tplJson.version;
      tplJson.description = description || tplJson.description;
      tplJson.keywords = keywords ? keywords.split(',') : tplJson.keywords;
      tplJson.author = author || tplJson.author;

      fs.writeFile(tplPath, JSON.stringify(tplJson, null, 2), 'utf-8', function(err) {
        if (err) {
          console.log('\nFailed to configure package.json file. ' +
            'Do not worry. ' +
            'The default value is used.');
          return;
        }
        console.log('\nConfigure the package.json file successfully!');
        resolve();
      });
    });
  });
};